/*****************************************************************************
 * FILE NAME: $RCSfile: gpio_example.c,v $      COPYRIGHT (c) FREESCALE 2004 *
 * DESCRIPTION:                                        All Rights Reserved   *
 * This is a sample MPC5500 program to show a top down approach to using     *
 * the eTPU.                                                                 *
 * It presents a simplifed idea for initializing the device in order to      *
 * concentrate on the eTPU. This first pass assumes an initialization tool   *
 * is not used.                                                              *
 *                                                                           *
 * This is a simple test program that sets up eTPU channels GPIO0 (4) and    *
 * GPIO1 (8) to to run the GPIO eTPU function - It requires a connection     *
 * between the channel pins to run correctly                                 *
 *===========================================================================*
 * ORIGINAL AUTHOR: Jeff Loeliger (r12110)                                   *
 * $Log: gpio_example.c,v $
 * Revision 1.1  2004/12/08 11:45:09  r47354
 * Updates as per QOM API rel_2_1
 * 
 *...........................................................................*
 * 0.1   J. Loeliger  05/Sep/03    Initial version.                          *
 * 0.2   K Terry      12/May/04    mod'd for test of GPIO                    *
 * 0.3   K Terry      03/Jun/04    mod'd to use fs_etpu_util functions       *
 * 0.4                             Updated for new build structure.          *
 * 0.4   G. Emerson   2/Nov/04     Remove paths from #includes               *
 ****************************************************************************/
#include "mpc5554.h"            /* mpc5554 register definitions */
#include "mpc5554_vars.h"       /* Variables specific to mpc5554 */
#include "etpu_util.h"          /* useful utility routines */
#include "fs_gpio.h"            /* GPIO API for the MPC5500 family */
#include "etpu_set1.h"          /* eTPU standard function set 1 */
#include "etpu_gpio.h"          /* eTPU GPIO API */

/* User written include files */
#include "gpio_example.h"       /*include application specific defines. */

int32_t pinstate;
int32_t error_status;

int32_t init_error;             /* initialization error code. */
uint32_t *fs_free_param;
uint32_t rate;

void delay (void);

main ()
{
    int32_t error_code;
    uint32_t rate;

    vuint16_t *upper_GPIO1_SCR16; /*address of upper 16 bits of GPIO1 SCR */
    vuint32_t *upper_GPIO1_SCR32; /*address of GPIO1 SCR */

    upper_GPIO1_SCR32 = &ETPU.CHAN[GPIO1].SCR.R;
    upper_GPIO1_SCR16 = (vuint16_t *) upper_GPIO1_SCR32;

    rate = 0x00000800;

    FMPLL.SYNCR.R = 0x06000000; /* MFD = 0b110, RFD=0b000, 8MHz*16=128MHz */

    /* initialize eTPU hardware */
    fs_etpu_init (my_etpu_config, (uint32_t *) etpu_code, sizeof (etpu_code),
                  (uint32_t *) etpu_globals, sizeof (etpu_globals));

    /* initialize eTPU channels */

    /* initialize channel GPIOO                           */

    error_code = fs_etpu_gpio_init (GPIO0, FS_ETPU_PRIORITY_MIDDLE);

    /* initialize channel GPIO1                          */

    error_code = fs_etpu_gpio_init (GPIO1, FS_ETPU_PRIORITY_MIDDLE);

    /* enable all timebases */
    fs_timer_start ();

    fs_gpio_config (FS_GPIO_ETPUA4,
                    FS_GPIO_PRIMARY_FUNCTION +
                    FS_GPIO_DRIVE_STRENGTH_50PF +
                    FS_GPIO_MAXIMUM_SLEW_RATE +
                    FS_GPIO_OUTPUT_DRAIN_DISABLE +
                    FS_GPIO_READBACK_ENABLE + FS_GPIO_OUTPUT_BUFFER_ENABLE);

    fs_gpio_config (FS_GPIO_ETPUA8,
                    FS_GPIO_PRIMARY_FUNCTION + FS_GPIO_INPUT_BUFFER_ENABLE);

    error_status = 0;

    *upper_GPIO1_SCR16 = 0xC0C0; /* clear DMA/interrupt service request */

/* eTPU Channel GPIO0 is set up as output and GPIO1 is configured for input
   GPIO0 should be connected externally to GPIO1 to allow test of function */

/* Test Input Immediate mode  and gpio_pin_history API function*/
    fs_etpu_gpio_output_low (GPIO0);
    /* GPIO0 configured as output low */
    delay ();
    fs_etpu_gpio_input_immed (GPIO1);
    while (ETPU.CHAN[GPIO1].SCR.B.CIS == 0);
    /* wait for eTPU_A Channel 1 interrupt status bit to be set */

    *upper_GPIO1_SCR16 = 0xC0C0;   /* clear DMA/interrupt service request */

    pinstate = fs_etpu_gpio_pin_history (GPIO1);
    delay ();
    if (pinstate != 0x000000)
      {
          while (1);
      }
    error_status++;
    fs_etpu_gpio_output_high (GPIO0);
    /* GPIO0 configured as output low */
    delay ();
    fs_etpu_gpio_input_immed (GPIO1);
    while (ETPU.CHAN[GPIO1].SCR.B.CIS == 0);
    /* wait for eTPU_A Channel 1 interrupt status bit to be set */

    *upper_GPIO1_SCR16 = 0xC0C0;  /* clear DMA/interrupt service request */

    pinstate = fs_etpu_gpio_pin_history (GPIO1);
    if (pinstate != 0x000001)
      {
          while (1);
      }
    error_status++;

    fs_etpu_gpio_output_low (GPIO0);
    /* GPIO0 configured as output low */

/* Test Input Match mode */

    fs_etpu_gpio_cfg_input_periodic (GPIO1, FS_ETPU_TCR1, rate);
    /* configure GPIO1 for input match mode */
    while (ETPU.CHAN[GPIO1].SCR.B.CIS == 0);
    /* wait for eTPU_A Channel 1 interrupt status bit to be set */

    *upper_GPIO1_SCR16 = 0xC0C0; /* clear DMA/interrupt service request */

    pinstate = fs_etpu_gpio_pin_history (GPIO1);
    pinstate = pinstate;
    if (pinstate != 0x000002)
      {
          while (1);
      }
    error_status++;

    fs_etpu_gpio_output_high (GPIO0);
    /* GPIO0 configured as output high */

    while (ETPU.CHAN[GPIO1].SCR.B.CIS == 0);
    /* wait for eTPU_A Channel 1 interrupt status bit to be set */

    *upper_GPIO1_SCR16 = 0xC0C0;  /* clear DMA/interrupt service request */

    pinstate = fs_etpu_gpio_pin_history (GPIO1);
    pinstate = pinstate;
    if (pinstate != 0x000005)
      {
          while (1);
      }
    error_status++;

    fs_etpu_gpio_output_low (GPIO0);
    /* GPIO0 configured as output low */

    while (ETPU.CHAN[GPIO1].SCR.B.CIS == 0);
    /* wait for eTPU_A Channel 1 interrupt status bit to be set */

    *upper_GPIO1_SCR16 = 0xC0C0; /* clear DMA/interrupt service request */

    pinstate = fs_etpu_gpio_pin_history (GPIO1);
    pinstate = pinstate;
    if (pinstate != 0x00000a)
      {
          while (1);
      }
    error_status++;

    fs_etpu_gpio_output_high (GPIO0);
    /* GPIO0 configured as output high */

    while (ETPU.CHAN[GPIO1].SCR.B.CIS == 0);
    /* wait for eTPU_A Channel 1 interrupt status bit to be set */

    *upper_GPIO1_SCR16 = 0xC0C0;  /* clear DMA/interrupt service request */

    pinstate = fs_etpu_gpio_pin_history (GPIO1);
    pinstate = pinstate;
    if (pinstate != 0x000015)
      {
          while (1);
      }
    error_status++;

    while (1)
      {
          fs_etpu_gpio_output_low (GPIO0);
          /* GPIO0 configured as output low */

          while (ETPU.CHAN[GPIO1].SCR.B.CIS == 0);
          /* wait for eTPU_A Channel 1 interrupt status bit to be set */

          *upper_GPIO1_SCR16 = 0xC0C0;  /* clear DMA/interrupt service request */
          fs_etpu_gpio_output_high (GPIO0);
          /* GPIO0 configured as output high */
          while (ETPU.CHAN[GPIO1].SCR.B.CIS == 0);
          /* wait for eTPU_A Channel 1 interrupt status bit to be set */
          *upper_GPIO1_SCR16 = 0xC0C0;  /* clear DMA/interrupt service request */
      }

}

void delay (void)
{
    volatile uint32_t ii;
    volatile uint32_t jj;
    for (ii = 0; ii < 0x200; ii++)
      {
          for (jj = 0; jj < 0x200; jj++);
      };
};
